package sun.nio.ch;

import java.util.LinkedList;
import java.util.HashMap;
import java.util.Iterator;
import java.nio.channels.MembershipKey;
import java.net.NetworkInterface;
import java.util.List;
import java.net.InetAddress;
import java.util.Map;

class MembershipRegistry
{
    private Map<InetAddress, List<MembershipKeyImpl>> groups;
    
    MembershipRegistry() {
        this.groups = null;
    }
    
    MembershipKey checkMembership(InetAddress inetAddress, NetworkInterface obj, InetAddress inetAddress2) {
        if (this.groups != null) {
            List<Object> list = (List)this.groups.get(inetAddress);
            if (list != null) {
                for (MembershipKeyImpl membershipKeyImpl : list) {
                    if (membershipKeyImpl.networkInterface().equals(obj)) {
                        if (inetAddress2 == null) {
                            if (membershipKeyImpl.sourceAddress() == null) {
                                return membershipKeyImpl;
                            }
                            throw new IllegalStateException("Already a member to receive all packets");
                        }
                        else {
                            if (membershipKeyImpl.sourceAddress() == null) {
                                throw new IllegalStateException("Already have source-specific membership");
                            }
                            if (inetAddress2.equals(membershipKeyImpl.sourceAddress())) {
                                return membershipKeyImpl;
                            }
                            continue;
                        }
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        return null;
    }
    
    void add(MembershipKeyImpl membershipKeyImpl) {
        InetAddress group = membershipKeyImpl.group();
        Object o;
        if (this.groups == null) {
            this.groups = new HashMap();
            o = null;
        }
        else {
            o = (List<MembershipKeyImpl>)this.groups.get(group);
        }
        if (o == null) {
            o = new LinkedList();
            this.groups.put(group, o);
        }
        o.add(membershipKeyImpl);
    }
    
    void remove(MembershipKeyImpl membershipKeyImpl) {
        InetAddress group = membershipKeyImpl.group();
        List<Object> list = (List)this.groups.get(group);
        if (list != null) {
            Iterator<Object> iterator = list.iterator();
            while (iterator.hasNext()) {
                if (iterator.next() != membershipKeyImpl) {
                    continue;
                }
                iterator.remove();
                break;
            }
            if (list.isEmpty()) {
                this.groups.remove(group);
            }
        }
    }
    
    void invalidateAll() {
        if (this.groups != null) {
            Iterator<InetAddress> iterator = this.groups.keySet().iterator();
            while (iterator.hasNext()) {
                Iterator<Object> iterator2 = ((List)this.groups.get((Object)iterator.next())).iterator();
                while (iterator2.hasNext()) {
                    ((MembershipKeyImpl)iterator2.next()).invalidate();
                }
            }
        }
    }
}
